// File             CubitInstrumentation.hpp
// Creator          Boyd Tidwell
// Created          2010
//
// Purpose          Collect and store usage data for cubit.

#ifndef CUBIT_INSTRUMENTATION_HPP
#define CUBIT_INSTRUMENTATION_HPP

#include <fstream>
#include <map>
#include <vector>
#include <string>
#include "CubitString.hpp"
#include "CubitUtilConfigure.h"

// NOTE: Since this header file is going to be used as input to a report generator, it
// is important that new tokens are added only to the end of the enum. 
enum UsageTokens
{
  GUI_BEGIN_TOKENS    = 0,
  GUI_CREATE_BLOCK    = 1,
  GUI_CREATE_NODESET  = 2,
  GUI_CREATE_SIDESET  = 3,
  GUI_CREATE_BC       = 4,
  GUI_CREATE_MATERIAL = 5,
  GUI_DRAW_BC         = 6,
  GUI_LIST_BC         = 7,
  GUI_MODIFY_BC       = 8,
  GUI_REMOVE_BC       = 9,
  GUI_EXPORT_MESH     = 10,
  GUI_EXPORT_METADATA = 11,
  GUI_SAVE_SCREENSHOT = 12,
  GUI_CURVE_CREATE    = 13,
  GUI_CURVE_DELETE    = 14,
  GUI_CURVE_MODIFY    = 15,
  GUI_CURVE_PREFS     = 16,
  GUI_CURVE_TRANSFORM = 17,
  GUI_DIRECTION_DIALOG    = 18,
  GUI_EXPORT_ACIS_DIALOG  = 19,
  GUI_GROUP_MANAGE        = 20,
  GUI_GROUP_WEBCUT        = 21,
  GUI_IMPORT_ACIS_DIALOG  = 22,
  GUI_LOCATION_DIALOG     = 23,
  GUI_PLANE_DIALOG        = 24,
  GUI_SURFACE_MERGE       = 25,
  GUI_SURFACE_CREATE      = 26,
  GUI_SURFACE_DELETE      = 27,
  GUI_SURFACE_MODIFY      = 28,
  GUI_SURFACE_PREFS       = 29,
  GUI_SURFACE_TRANSFORM   = 30,
  GUI_VERTEX_CREATE       = 31,
  GUI_VERTEX_DELETE       = 32,
  GUI_VERTEX_MODIFY       = 33,
  GUI_VERTEX_PREFS        = 34,
  GUI_VOLUME_CREATE       = 35,
  GUI_VOLUME_DELETE       = 36,
  GUI_VOLUME_MODIFY       = 37,
  GUI_VOLUME_BOOLEAN      = 38,
  GUI_VOLUME_IMPRINT      = 39,
  GUI_VOLUME_PREFS        = 40,
  GUI_VOLUME_TRANSFORM    = 41,
  GUI_VOLUME_WEBCUT       = 42,
  GUI_TOD_NEXT            = 43,
  GUI_TOD_PREVIOUS        = 44,
  GUI_TOD_CLOSE           = 45,
  GUI_TOD_DONT_SHOW       = 46,
  GUI_CREATE_MESH_ELEMENT = 47,
  GUI_CURVE_MESH          = 48,
  GUI_CURVE_MESH_COPY     = 49,
  GUI_CURVE_SMOOTH        = 50,
  GUI_DELETE_MESH_ELEMENT = 51,
  GUI_EDGE_SWAP           = 52,
  GUI_NODE_EQUIVALENCE    = 53,
  GUI_HELP_BUTTON         = 54,
  GUI_GROUP_MESH_SCHEME   = 55,
  GUI_GROUP_SMOOTH        = 56,
  GUI_MESH_SMOOTH         = 57,
  GUI_IMPORT_GRANITE      = 58,
  GUI_IMPORT_MESH         = 59,
  GUI_NODE_MERGE          = 60,
  GUI_MESH_BOUNDARY_SURFACE = 61,
  GUI_MESH_CONTROL_SKEW     = 62,
  GUI_MESH_DELETE           = 63,
  GUI_MESH_ENTITIES         = 64,
  GUI_MESH_INTERVAL         = 65,
  GUI_MESH_MODIFY           = 66,
  GUI_MESH_SKIN             = 67,
  GUI_MESH_TRANSFORM        = 68,
  GUI_NODE_MOVE             = 69,
  GUI_RENUMBER_MESH_ELEMENT = 70,
  GUI_SURFACE_MESH_COPY     = 71,
  GUI_SURFACE_MESH_MODIFY   = 72,
  GUI_SURFACE_MESH_QUALITY  = 73,
  GUI_SURFACE_MESH_REFINE   = 74,
  GUI_SURFACE_MESH_SMOOTH   = 75,
  GUI_TET_CLEANUP           = 76,
  GUI_TET_MESH              = 77,
  GUI_TET_MESH_MATCH        = 78,
  GUI_TET_REMESH            = 79,
  GUI_VERTEX_MESH           = 80,
  GUI_VOLUME_MESH_COPY      = 81,
  GUI_VOLUME_MESH_MODIFY    = 82,
  GUI_VOLUME_MESH_QUALITY   = 83,
  GUI_VOLUME_MESH_REFINE    = 84,
  GUI_VOLUME_MESH_SMOOTH    = 85,
  GUI_VOLUME_TOLERANT_MESH  = 86,
  GUI_EXPORT_EXODUS         = 87,
  GUI_EXPORT_STL            = 88,
  GUI_EXPORT_PRESTO         = 89,
  GUI_EXPORT_FACETS         = 90,
  GUI_FILEMENU_NEW          = 91,
  GUI_FILEMENU_OPEN         = 92,
  GUI_FILEMENU_SAVE         = 93,
  GUI_FILEMENU_OPEN_CUB     = 94,
  GUI_FILEMENU_IMPORT_CATIA = 95,
  GUI_FILEMENU_EXPORT_CATIA = 96,
  GUI_EDITMENU_UNDO         = 97,
  GUI_TOOLBAR_PICK_BODY     = 98,
  GUI_TOOLBAR_PICK_VOLUME   = 99,
  GUI_TOOLBAR_PICK_SURFACE  = 100,
  GUI_TOOLBAR_PICK_CURVE    = 101,
  GUI_TOOLBAR_PICK_VERTEX   = 102,
  GUI_TOOLBAR_PICK_GROUP    = 103,
  GUI_TOOLBAR_PICK_NODE   = 104,
  GUI_TOOLBAR_PICK_EDGE   = 105,
  GUI_TOOLBAR_PICK_FACE   = 106,
  GUI_TOOLBAR_PICK_TRI    = 107,
  GUI_TOOLBAR_PICK_TET    = 108,
  GUI_TOOLBAR_PICK_HEX    = 109,
  GUI_TOOLBAR_PICK_ACCEL  = 110,
  GUI_TOOLBAR_PICK_VELOC  = 111,
  GUI_TOOLBAR_PICK_FORCE  = 112,
  GUI_TOOLBAR_PICK_PRESS  = 113,
  GUI_TOOLBAR_PICK_HEATF  = 114,
  GUI_TOOLBAR_PICK_DISPL  = 115,
  GUI_TOOLBAR_PICK_TEMP   = 116,
  GUI_TOOLBAR_PICK_CONV   = 117,
  GUI_TOOLBAR_PICK_INMFL  = 118,
  GUI_TOOLBAR_PICK_INPRES = 119,
  GUI_TOOLBAR_PICK_INVEL  = 120,
  GUI_TOOLBAR_PICK_OUTPRES  = 121,
  GUI_TOOLBAR_PICK_FFPRES   = 122,
  GUI_TOOLBAR_PICK_SYMMT    = 123,
  GUI_TOOLBAR_PICK_WALL     = 124,
  GUI_TOOLBAR_PICK_NODESET  = 125,
  GUI_TOOLBAR_PICK_SIDESET  = 126,
  GUI_TOOLBAR_ZOOM_IN       = 127,
  GUI_TOOLBAR_ZOOM_OUT      = 128,
  GUI_TOOLBAR_ZOOM_RESET    = 129,
  GUI_TOOLBAR_TOGGLE_SCALE  = 130,
  GUI_TOOLBAR_TOGGLE_PERSP  = 131,
  GUI_TOOLBAR_FORCE_SCALE_SYNC  = 132,
  GUI_TOOLBAR_TOGGLE_CLIP       = 133,
  GUI_TOOLBAR_TOGGLE_CLIP_MANIP = 134,
  GUI_TOOLBAR_SHOW_VALENCE      = 135,
  GUI_TOOLBAR_SELECT_ENCLOSED = 136,
  GUI_TOOLBAR_SELECT_XRAY     = 137,
  GUI_TOOLBAR_SELECT_BOX      = 138,
  GUI_SHOW_GEOM_POPUP         = 139,
  GUI_SHOW_MESH_POPUP         = 140,
  GUI_SHOW_DEFAULT_POPUP      = 141,
  GUI_PICK_NEXT_DIALOG        = 142,
  GUI_TREE_DROP_GROUP         = 143,
  GUI_TREE_DROP_BC            = 144,
  GUI_TREE_DROP_EXODUS        = 145,
  GUI_TREE_DROP_METADATA      = 146,
  GUI_TREE_CREATE_METADATA    = 147,
  GUI_TREE_CREATE_EXODUS      = 148,
  GUI_TREE_MODIFY_METADATA    = 149,
  GUI_TREE_RENAME_ENTITY      = 150,
  GUI_TREE_RENAME_EXODUS      = 151,
  GUI_TREE_RENAME_BC          = 152,
  GUI_TREE_DRAW               = 153,
  GUI_TREE_IMPORT             = 154,
  GUI_TREE_ZOOM_TO            = 155,
  GUI_TREE_FLY_INTO           = 156,
  GUI_TREE_LOCATE             = 157,
  GUI_TREE_GOTO_PART          = 158,
  GUI_TREE_ISOLATE            = 159,
  GUI_TREE_DELETE_ENTITY      = 160,
  GUI_TREE_ROTATE_ABOUT       = 161,
  GUI_TREE_VIS_ON             = 162,
  GUI_TREE_MEASURE            = 163,
  GUI_TREE_MESH_SELECTED      = 164,
  GUI_TREE_DELETE_MESH        = 165,
  GUI_TREE_LIST_QUALITY       = 166,
  GUI_TREE_SWAP_EDGE          = 167,
  GUI_TREE_MEASURE_VOID       = 168,
  GUI_TREE_REMOVE_FROM_GROUP  = 169,
  GUI_TREE_CLEANOUT_GROUP     = 170,
  GUI_TREE_CREATE_GROUP       = 171,
  GUI_TREE_VIEW_DESCENDENTS   = 172,
  GUI_TREE_VIEW_ANCESTORS     = 173,
  GUI_TREE_VIEW_NEIGHBORS     = 174,
  GUI_TREE_VIEW_SWEEP_SURFACES  = 175,
  GUI_TREE_LIST                 = 176,
  GUI_TREE_CREATE_VOLUME        = 177,
  GUI_TREE_CLEANUP_VOLUME       = 178,
  GUI_TREE_REMESH_VOLUME        = 179,
  GUI_TREE_RESET_VOL_SURF       = 180,
  GUI_TREE_CREATE_MATERIAL      = 181,
  GUI_TREE_CREATE_MEDIA         = 182,
  GUI_TREE_CREATE_BLOCK         = 183,
  GUI_TREE_CREATE_SIDESET       = 184,
  GUI_TREE_CREATE_NODESET       = 185,
  GUI_TREE_CREATE_BC            = 186,
  GUI_TREE_MODIFY_BC            = 187,
  GUI_TREE_CULL_SIDESET         = 188,
  GUI_TREE_DRAW_SIDESET         = 189,
  GUI_TREE_LIST_SIDESET         = 190,
  GUI_TREE_DELETE_BC            = 191,
  GUI_TREE_HIGHLIGHT_BC         = 192,
  GUI_TREE_DRAW_BC              = 193,
  GUI_TREE_LIST_BC              = 194,
  GUI_TREE_DRAW_BLOCK           = 195,
  GUI_TREE_LIST_BLOCK           = 196,
  GUI_TREE_CLEANUP_BLOCK        = 197,
  GUI_TREE_REMESH_BLOCK         = 198,
  GUI_TREE_DRAW_NODESET         = 199,
  GUI_TREE_LIST_NODESET         = 200,
  GUI_TREE_REMOVE_FROM_BLOCK    = 201,
  GUI_TREE_REMOVE_FROM_SIDESET  = 202,
  GUI_TREE_REMOVE_FROM_NODESET  = 203,
  GUI_TREE_CREATE_ASSEMBLY      = 204,
  GUI_TREE_CREATE_SUB_ASSEMBLY  = 205,
  GUI_TREE_CREATE_PART          = 206,
  GUI_TREE_REMOVE_FROM_METADATA = 207,
  GUI_TREE_LIST_METADATA      = 208,
  GUI_TREE_CLEAN_METADATA     = 209,
  GUI_TREE_LIST_VOLS_WO_PARTS = 210,
  GUI_TREE_SHOW_PART_NAME     = 211,
  GUI_TREE_SHOW_PART_DESC     = 212,
  GUI_TREE_REMOVE_MESH_FROM_GROUPS = 213,
  GUI_GRT_DRAW_ENTITY         = 214,
  GUI_GRT_MERGE_ALL           = 215,
  GUI_GRT_HEAL                = 216,
  GUI_GRT_REGULARIZE          = 217,
  GUI_GRT_MINIMIZE_FEATURES   = 218,
  GUI_GRT_FLY_INTO            = 219,
  GUI_GRT_ZOOM_TO             = 220,
  GUI_GRT_ZOOM_RESET          = 221,
  GUI_GRT_LOCATE              = 222,
  GUI_GRT_HIGHLIGHT_ALL       = 223,
  GUI_GRT_REMOVE_SMALL_SURFACES = 224,
  GUI_GRT_REMOVE_ENTITY         = 225,
  GUI_GRT_COMPOSITE             = 226,
  GUI_GRT_REMOVE_SLIVERS        = 227,
  GUI_GRT_AUTO_CLEAN            = 228,
  GUI_GRT_SPLIT_SURFACE         = 229,
  GUI_GRT_TWEAK                 = 230,
  GUI_GRT_COLLAPSE_ANGLE        = 231,
  GUI_GRT_COLLAPSE_CURVE        = 232,
  GUI_GRT_COLLAPSE_SURFACE      = 233,
  GUI_GRT_GET_SMALL_SURFACES    = 234,
  GUI_GRT_GET_SMALL_CURVES      = 235,
  GUI_GRT_GET_SMALL_VOLUMES     = 236,
  GUI_GRT_GET_MERGEABLE_ENTITIES  = 237,
  GUI_GRT_GET_CLOSE_LOOPS         = 238,
  GUI_GRT_GET_BLENDS              = 239,
  GUI_GRT_GET_TANGENTS            = 240,
  GUI_GRT_GET_BAD_CURVE_ANGLES    = 241,
  GUI_GRT_GET_BAD_SURFACE_ANGLES  = 242,
  GUI_GRT_GET_BAD_GEOMETRY        = 243,
  GUI_GRT_GET_OVERLAPPING_SURFACES  = 244,
  GUI_GRT_GET_OVERLAPPING_VOLUMES   = 245,
  GUI_MPT_UNMERGE             = 246,
  GUI_MPT_MESH_SELECTED       = 247,
  GUI_MPT_DELETE_MESH         = 248,
  GUI_MPT_IMPRINT_MERGE       = 249,
  GUI_MPT_DRAW                = 250,
  GUI_MPT_VIS_ON              = 251,
  GUI_MPT_VIS_OFF             = 252,
  GUI_MPT_RESET_GRAPHICS      = 253,
  GUI_MPT_DRAW_TREE           = 254,
  GUI_MPT_RESTORE_COLORS      = 255,
  GUI_MPT_COLOR_SURFACES      = 256,
  GUI_MPT_ZOOM_TO             = 257,
  GUI_MPT_LOCATE              = 258,
  GUI_MPT_ROTATE_ABOUT        = 259,
  GUI_MPT_VIEW_DESCENDENTS    = 260,
  GUI_MPT_VIEW_ANCESTORS      = 261,
  GUI_MPT_VIEW_NEIGHBORS      = 262,
  GUI_MPT_ANALYZE_MESH        = 263,
  GUI_MPT_WEBCUT              = 264,
  GUI_MPT_SHOW_SIZE_PANEL     = 265,
  GUI_MPT_SHOW_SCHEME_PANEL   = 266,
  GUI_MPT_SHOW_VERTEX_TYPES_PANEL = 267,
  GUI_MQT_SMOOTH_PARENT           = 268,
  GUI_MQT_SMOOTH                  = 269,
  GUI_MQT_DELETE_ENTITY           = 270,
  GUI_MQT_DELETE_MESH             = 271,
  GUI_MQT_MOVE_NODE               = 272,
  GUI_MQT_MERGE_NODE              = 273,
  GUI_MQT_VALIDATE_MESH           = 274,
  GUI_MQT_CHECK_COINCIDENT_NODES  = 275,
  GUI_MQT_VIS_OFF             = 276,
  GUI_MQT_FLY_INTO            = 277,
  GUI_MQT_PROCESS_COLOR_CODE  = 278,
  GUI_MQT_CLEANUP_BLOCK       = 279,
  GUI_MQT_LIST_BLOCK          = 280,
  GUI_MQT_DRAW                = 281,
  GUI_MQT_ROTATE_ABOUT        = 282,
  GUI_MQT_VIS_ON              = 283,
  GUI_MQT_SMOOTH_1_LAYER      = 284,
  GUI_MQT_SMOOTH_2_LAYERS     = 285,
  GUI_MQT_REMESH_2_TET_LAYERS = 286,
  GUI_MQT_REMESH_VOLUME       = 287,
  GUI_MQT_SHOW_SMOOTH_SURF_PANEL  = 288,
  GUI_MQT_REFINE                  = 289,
  GUI_MQT_ANALYZE_QUALITY         = 290,
  GUI_ITEM_BC_BLOCK               = 291,
  GUI_ITEM_BC_SIDESET             = 292,
  GUI_ITEM_BC_NODESET             = 293,
  GUI_ITEM_BC_OTHER               = 294,
  GUI_ITEM_BC_MATERIAL            = 295,
  GUI_ITEM_CLEAN_RUN_ALL_DIAGNOSTICS  = 296,
  GUI_ITEM_CLEAN_MARK_OKAY_ALL    = 297,
  GUI_ITEM_CLEAN_UNMARK_OKAY_ALL  = 298,
  GUI_ITEM_CLEAN_MARK_OKAY        = 299,
  GUI_ITEM_CLEAN_MARK_SURFACE_PAIR_OKAY   = 300,
  GUI_ITEM_CLEAN_UNMARK_SURFACE_PAIR_OKAY = 301,
  GUI_ITEM_CLEAN_FIX_INVALID_TOPOLOGY     = 302,
  GUI_ITEM_CLEAN_REMOVE_SMALL_FEATURES    = 303,
  GUI_ITEM_CLEAN_CONNECT_VOLUMES          = 304,
  GUI_ITEM_CLEAN_BUILD_MESHABLE_TOPOLOGY  = 305,
  GUI_ITEM_CLEAN_SET_ELEMENT_SIZES        = 306,
  GUI_ITEM_CLEAN_RUN_HEALER               = 307,
  GUI_ITEM_CLEAN_AUTO_HEAL                = 308,
  GUI_ITEM_CLEAN_SHOW_SMALL_FEATURES      = 309,
  GUI_ITEM_CLEAN_AUTO_REPAIR_SMALL_FEATURES       = 310,
  GUI_ITEM_CLEAN_EXECUTE_SMALL_FEATURES_SOLUTION  = 311,
  GUI_ITEM_CLEAN_AUTO_CONNECT             = 312,
  GUI_ITEM_CLEAN_ADD_CONTACT_SURFACE      = 313,
  GUI_ITEM_CLEAN_REMOVE_AS_CONTACT        = 314,
  GUI_ITEM_CLEAN_DETECT_BLENDS            = 315,
  GUI_ITEM_CLEAN_MODIFY_BLENDS            = 316,
  GUI_ITEM_CLEAN_EXECUTE_BLENDS_SOLUTION  = 317,
  GUI_ITEM_CLEAN_EXECUTE_DECOMP_SOLUTION  = 318,
  GUI_ITEM_CLEAN_UPDATE_SOURCE_TARGET     = 319,
  GUI_ITEM_CLEAN_EXECUTE_SOURCE_TARGET_SOLUTION = 320,
  GUI_ITEM_CLEAN_UPDATE_FORCE_SWEEP       = 321,
  GUI_ITEM_CLEAN_REPAIR_FORCE_SWEEP       = 322,
  GUI_ITEM_CLEAN_EXECUTE_FORCE_SWEEP_SOLUTION = 323,
  GUI_ITEM_CLEAN_DETECT_OVERLAPPING_VOLUMES   = 324,
  GUI_ITEM_CLEAN_EXECUTE_OVERLAPPING_VOLUMES_SOLUTION = 325,
  GUI_ITEM_CLEAN_CHANGE_IMPRINT_TOLERANCE     = 326,
  GUI_ITEM_CLEAN_IMPRINT_MERGE                = 327,
  GUI_ITEM_CLEAN_EXECUTE_OVERLAPPING_SOLUTIONS = 328,
  GUI_ITEM_CLEAN_DETECT_OVERLAPPING_SURFACES    = 329,
  GUI_ITEM_CLEAN_EXECUTE_IMPRINT_MERGE_SOLUTION = 330,
  GUI_ITEM_CLEAN_AUTOFIX_ALL                = 331,
  GUI_ITEM_CLEAN_FIND_SMALL_FEATURES        = 332,
  GUI_ITEM_CLEAN_ESTIMATE_MERGE_TOLERANCE   = 333,
  GUI_ITEM_CLEAN_DETERMINE_MERGE_TOLERANCE  = 334,
  GUI_ITEM_EXPORT                           = 335,
  GUI_ITEM_MESH_GENERATE_MESH               = 336,
  GUI_ITEM_SETUP_DENSITY_SLIDER_MOVED       = 337,
  GUI_ITEM_SETUP_HEX_ELEMENT_TYPE           = 338,
  GUI_ITEM_SETUP_TET_ELEMENT_TYPE           = 339,
  GUI_ITEM_SETUP_ELEMENT_BUDGET             = 340,
  GUI_ITEM_SETUP_APPLY_ELEMENT_SIZE         = 341,
  GUI_ITEM_SETUP_PREVIEW_MESH               = 342,
  GUI_ITEM_SETUP_DEFAULT_SETTINGS           = 343,
  GUI_ITEM_SETUP_APPLY_BUDGET               = 344,
  GUI_ITEM_SETUP_APPLY_SIZE                 = 345,
  GUI_ITEM_VALIDATE_COLOR_CODE              = 346,
  GUI_ITEM_VALIDATE_ANALYZE                 = 347,
  GUI_ITEM_VALIDATE_VALIDATE_MESH           = 348,
  GUI_ITEM_VALIDATE_CHECK_COINCIDENT_NODES  = 349,
  GUI_ITEM_VALIDATE_DELETE_MESH             = 350,
  GUI_ITEM_VALIDATE_EXECUTE_SOLUTIONS       = 351,
  GUI_OPTIONS_MOUSE_EMULATE                 = 352,
  GUI_OPTIONS_POST_PROCESSER_SETUP          = 353,
  GUI_OPTIONS_DISPLAY_TRIAD_TOGGLED         = 354,
  GUI_OPTIONS_DISPLAY_HIGHLIGHT_SURFACES_TOGGLED  = 355,
  GUI_OPTIONS_DISPLAY_PERSPECTIVE_ANGLE_CHANGED   = 356,
  GUI_OPTIONS_DISPLAY_LINEWIDTH_CHANGED           = 357,
  GUI_OPTIONS_DISPLAY_HIGHLIGHT_LINEWIDTH_CHANGED = 358,
  GUI_OPTIONS_DISPLAY_TEXTSIZE_CHANGED            = 359,
  GUI_OPTIONS_DISPLAY_AMBIENT_INTENSITY_CHANGED   = 360,
  GUI_OPTIONS_DISPLAY_LIGHT_INTENSITY_CHANGED     = 361,
  GUI_OPTIONS_DISPLAY_AMBIENT_COLOR_CHANGED       = 362,
  GUI_OPTIONS_DISPLAY_LIGHT_COLOR_CHANGED         = 363,
  GUI_OPTIONS_DISPLAY_BACKGROUND_COLOR_CHANGED    = 364,
  GUI_OPTIONS_DISPLAY_TOGGLE_PRESELECT            = 365,
  GUI_OPTIONS_GEOMETRY_VERTEX_SIZE                = 366,
  GUI_OPTIONS_GEOMETRY_DEFAULT_FACETS             = 367,
  GUI_OPTIONS_GEOMETRY_FACET_ANGLE_CHANGED        = 368,
  GUI_OPTIONS_GEOMETRY_FACET_TOLERANCE_CHANGED    = 369,
  GUI_OPTIONS_GEOMETRY_GRAPHICS_SILHOUETTE_TOGGLED  = 370,
  GUI_OPTIONS_GEOMETRY_GRAPHICS_SILHOUETTE_PATTERN  = 371,
  GUI_OPTIONS_GEOMETRY_ACIS_ENGINE                  = 372,
  GUI_OPTIONS_GEOMETRY_FACET_ENGINE                 = 373,
  GUI_OPTIONS_GEOMETRY_OCC_ENGINE                   = 374,
  GUI_OPTIONS_GEOMETRY_CATIA_ENGINE                 = 375,
  GUI_OPTIONS_GEOMETRY_MESH_AUTODELETE_TOGGLED      = 376,
  GUI_OPTIONS_HEX_QUALITY_CHANGED                   = 377,
  GUI_OPTIONS_HEX_QUALITY_DEFAULT                   = 378,
  GUI_OPTIONS_MESH_LINE_COLOR                       = 379,
  GUI_OPTIONS_MESH_NODE_SIZE_CHANGED                = 380,
  GUI_OPTIONS_MESH_GRAPHICS_SHRINK                  = 381,
  GUI_OPTIONS_MESH_HEX_ELEMENT                      = 382,
  GUI_OPTIONS_MESH_TRI_ELEMENT                      = 383,
  GUI_OPTIONS_MESH_CHANGE_MAP_COLOR                 = 384,
  GUI_OPTIONS_QUAD_QUALITY_CHANGED                  = 385,
  GUI_OPTIONS_QUAD_QUALITY_DEFAULT                  = 386,
  GUI_OPTIONS_TET_QUALITY_CHANGED                   = 387,
  GUI_OPTIONS_TET_QUALITY_DEFAULT                   = 388,
  GUI_PP_MESH_ENTITY        = 389,
  GUI_PP_SMOOTH_ENTITY      = 390,
  GUI_PP_CHECK_QUALITY      = 391,
  GUI_PP_PREVIEW_MESH       = 392,
  GUI_PP_DELETE_MESH        = 393,
  GUI_PP_RESET_ENTITY       = 394,
  GUI_PP_MODIFY_ACCELERATION  = 395,
  GUI_PP_MODIFY_METADATA      = 396,
  GUI_PP_REMOVE_METADATA      = 397,
  GUI_PP_MODIFY_BC            = 398,
  GUI_PP_DELETE_BC            = 399,
  GUI_PP_BLOCK_ELEMENT_TYPE   = 400,
  GUI_PP_BLOCK_ATTRIBUTE_COUNT  = 401,
  GUI_PP_BLOCK_ATTRIBUTE_INDEX  = 402,
  GUI_PP_CFD_CHANGED            = 403,
  GUI_PP_CONVECTION_MODIFY      = 404,
  GUI_PP_DISPLACEMENT_MODIFY    = 405,
  GUI_PP_EXODUS_MODIFY          = 406,
  GUI_PP_FORCE_MODIFY           = 407,
  GUI_PP_GEOMETRY_MODIFY        = 408,
  GUI_PP_GEOMETRY_MESH_SCHEME   = 409,
  GUI_PP_GEOMETRY_INTERVAL      = 410,
  GUI_PP_GEOMETRY_SIZE          = 411,
  GUI_PP_GEOMETRY_HEATFLUX_MODIFY = 412,
  GUI_PP_GEOMETRY_MATERIAL_MODIFY = 413,
  GUI_PP_GEOMETRY_MEDIA_MODIFY    = 414,
  GUI_PP_GEOMETRY_PART_MODIFY     = 415,
  GUI_PP_GEOMETRY_PRESSURE_MODIFY = 416,
  GUI_PP_GEOMETRY_SURFACE_SMOOTH_SCHEME = 417,
  GUI_PP_GEOMETRY_TEMPERATURE_MODIFY    = 418,
  GUI_PP_TOP_ASSEMBLY_MODIFY    = 419,
  GUI_PP_VELOCITY_MODIFY        = 420,
  GUI_PP_VOLUME_SMOOTH_SCHEME   = 421,
  GUI_PP_VOLUME_DELETE          = 422,
  GUI_PP_CONTACT_PAIR_MODIFY    = 423,
  GUI_PP_CONTACT_REGION_MODIFY  = 424,
  GUI_TOOLBAR_PICK_CONTACT_REGION   = 425,
  GUI_TOOLBAR_PICK_CONTACT_PAIR     = 426,
  GUI_VERTEX_TRANSFORM = 427,
  GUI_TOOLBAR_PICK_BLOCK  = 428,
  GUI_OPTIONS_DISPLAY_GRAPHICS_AXIS_TOGGLED         = 429,

  GUI_END_TOKENS
};


class CUBIT_UTIL_EXPORT CubitInstrumentation
{

public:
  static CubitInstrumentation* instance();
  static void delete_instance();

  virtual ~CubitInstrumentation();

    // write a single token
  void write_context_token(int token);

    // write a number of keywords
  void write_keywords(std::vector<CubitString> keywords);

  void load_all_words(const char* const allwords[]);

  void record_tokens(bool t_or_f);
  bool record_tokens();

  bool valid_keywords();

protected:
  static CubitInstrumentation* thisInstance;

  CubitInstrumentation();

private:
  enum SectionState{Unknown, Command, GUI, Keyword} outputState;
  bool recordTokens;
  bool validKeywords;
  bool writeKeywords;
  std::ofstream *tokenUsageStream;
  std::map<std::string, int> keywordMap;

  bool check_token_log();

  void close_section();
  SectionState open_section(SectionState state);

  int lookup_keyword(const char* keyword);
  void write_all_words();
};

inline void CubitInstrumentation::record_tokens(bool t_or_f)
{recordTokens = t_or_f;}

inline bool CubitInstrumentation::record_tokens()
{return recordTokens;}

inline bool CubitInstrumentation::valid_keywords()
{return validKeywords;}

#endif


